import * as unctx from 'unctx';

interface ConfigSchema {
   /** Vue.js configuration */
  vue: {
    config: {
       /** @default true */
       silent: boolean,

       /** @default true */
       performance: boolean,
    },
  },

  app: {
    [key: string]: any
  },

  /**
   * Uses {srcDir}/app.html if exists by default otherwise nuxt default
   * @default "/home/pooya/tmp/framework/packages/kit/<rootDir>/.nuxt/views/app.template.html"
  */
  appTemplatePath: string,

  /** @default false */
  store: boolean,

  debug: any,

  /** @default null */
  vueMeta: any,

  head: {
    meta: Array<any>,

    link: Array<any>,

    style: Array<any>,

    script: Array<any>,
  },

  fetch: {
    /** @default true */
    server: boolean,

    /** @default true */
    client: boolean,
  },

  plugins: Array<any>,

  /** @default null */
  extendPlugins: any,

  css: Array<any>,

  layouts: any,

  /** @default null */
  ErrorPage: any,

  loading: {
    /** @default "black" */
    color: string,

    /** @default "red" */
    failedColor: string,

    /** @default "2px" */
    height: string,

    /** @default 200 */
    throttle: number,

    /** @default 5000 */
    duration: number,

    /** @default false */
    continuous: boolean,

    /** @default false */
    rtl: boolean,

    /** @default true */
    css: boolean,
  },

  loadingIndicator: {
    [key: string]: any
  },

  pageTransition: {
    /** @default "page" */
    name: string,

    /** @default "out-in" */
    mode: string,

    /** @default false */
    appear: boolean,

    /** @default "appear" */
    appearClass: string,

    /** @default "appear-active" */
    appearActiveClass: string,

    /** @default "appear-to" */
    appearToClass: string,
  },

  layoutTransition: {
    /** @default "layout" */
    name: string,

    /** @default "out-in" */
    mode: string,
  },

  features: {
    /** @default true */
    store: boolean,

    /** @default true */
    layouts: boolean,

    /** @default true */
    meta: boolean,

    /** @default true */
    middleware: boolean,

    /** @default true */
    transitions: boolean,

    /** @default true */
    deprecations: boolean,

    /** @default true */
    validate: boolean,

    /** @default true */
    asyncData: boolean,

    /** @default true */
    fetch: boolean,

    /** @default true */
    clientOnline: boolean,

    /** @default true */
    clientPrefetch: boolean,

    /** @default true */
    componentAliases: boolean,

    /** @default true */
    componentClientOnly: boolean,
  },

  /** @default "/home/pooya/tmp/framework/packages/kit/<rootDir>" */
  rootDir: string,

  /** @default "/home/pooya/tmp/framework/packages/kit/<rootDir>" */
  srcDir: string,

  /** @default "/home/pooya/tmp/framework/packages/kit/<rootDir>/.nuxt" */
  buildDir: string,

  /** @default true */
  dev: boolean,

  /** @default false */
  test: boolean,

  env: {
    [key: string]: any
  },

  createRequire: () => any,

  /** @default "server" */
  target: string,

  /** @default true */
  ssr: boolean,

  /**
   * @default "spa"
   * 
   * @deprecated `mode` option is deprecated
  */
  mode: string,

  modern: any,

  modules: Array<any>,

  buildModules: Array<any>,

  _modules: Array<any>,

  /** @default "nuxt" */
  globalName: string,

  globals: {
    id: () => any,

    nuxt: () => any,

    context: () => any,

    pluginPrefix: () => any,

    readyCallback: () => any,

    loadedCallback: () => any,
  },

  serverMiddleware: Array<any>,

  /** @default ["/home/pooya/tmp/framework/packages/kit/<rootDir>/node_modules","/home/pooya/tmp/framework/packages/kit/node_modules"] */
  modulesDir: Array<string>,

  dir: {
    /** @default "assets" */
    assets: string,

    /** @default "app" */
    app: string,

    /** @default "layouts" */
    layouts: string,

    /** @default "middleware" */
    middleware: string,

    /** @default "pages" */
    pages: string,

    /** @default "static" */
    static: string,

    /** @default "store" */
    store: string,
  },

  /** @default [".js",".mjs",".ts",".tsx",".vue"] */
  extensions: Array<string>,

  /** @default [".css",".pcss",".postcss",".styl",".stylus",".scss",".sass",".less"] */
  styleExtensions: Array<string>,

  alias: {
    [key: string]: any
  },

  ignoreOptions: any,

  /** @default "-" */
  ignorePrefix: string,

  /** @default ["**\/*.test.*","**\/*.spec.*","**\/-*.*"] */
  ignore: Array<string>,

  watch: Array<any>,

  watchers: {
    rewatchOnRawEvents: any,

    webpack: {
       /** @default 1000 */
       aggregateTimeout: number,
    },

    chokidar: {
       /** @default true */
       ignoreInitial: boolean,
    },
  },

  editor: any,

  /** @default null */
  hooks: any,

  privateRuntimeConfig: any,

  publicRuntimeConfig: {
    app: {
       [key: string]: any
    },
  },

  /** @default 2 */
  _majorVersion: number,

  /** @default false */
  _legacyGenerate: boolean,

  /** @default false */
  _start: boolean,

  /** @default false */
  _build: boolean,

  /** @default false */
  _generate: boolean,

  /** @default false */
  _cli: boolean,

  _requiredModules: any,

  _nuxtConfigFile: any,

  _nuxtConfigFiles: Array<any>,

  /** @default "" */
  appDir: string,

  /** @default false */
  vite: boolean,

  build: {
    /** @default false */
    quiet: boolean,

    /** @default false */
    analyze: boolean,

    /** @default false */
    profile: boolean,

    /** @default false */
    extractCSS: boolean,

    /** @default true */
    cssSourceMap: boolean,

    ssr: any,

    /** @default false */
    parallel: boolean,

    /** @default false */
    cache: boolean,

    /** @default false */
    standalone: boolean,

    /** @default "/_nuxt/" */
    publicPath: string,

    /** @default "url" */
    serverURLPolyfill: string,

    filenames: {
       app: () => any,

       chunk: () => any,

       css: () => any,

       img: () => any,

       font: () => any,

       video: () => any,
    },

    loaders: {
       file: {
           /** @default false */
           esModule: boolean,
       },

       fontUrl: {
           /** @default false */
           esModule: boolean,

           /** @default 1000 */
           limit: number,
       },

       imgUrl: {
           /** @default false */
           esModule: boolean,

           /** @default 1000 */
           limit: number,
       },

       pugPlain: any,

       vue: {
           /** @default true */
           productionMode: boolean,

           transformAssetUrls: {
                /** @default "src" */
                video: string,

                /** @default "src" */
                source: string,

                /** @default "src" */
                object: string,

                /** @default "src" */
                embed: string,
           },
       },

       css: {
           /** @default 0 */
           importLoaders: number,

           /** @default false */
           esModule: boolean,
       },

       cssModules: {
           /** @default 0 */
           importLoaders: number,

           /** @default false */
           esModule: boolean,

           modules: {
                /** @default "[local]_[hash:base64:5]" */
                localIdentName: string,
           },
       },

       less: any,

       sass: {
           sassOptions: {
                /** @default true */
                indentedSyntax: boolean,
           },
       },

       scss: any,

       stylus: any,

       vueStyle: any,
    },

    styleResources: any,

    plugins: Array<any>,

    terser: any,

    /** @default false */
    hardSource: boolean,

    /** @default false */
    aggressiveCodeRemoval: boolean,

    /** @default false */
    optimizeCSS: boolean,

    optimization: {
       /** @default "single" */
       runtimeChunk: string,

       /** @default true */
       minimize: boolean,

       minimizer: any,

       splitChunks: {
           /** @default "all" */
           chunks: string,

           /** @default "/" */
           automaticNameDelimiter: string,

           cacheGroups: any,
       },
    },

    splitChunks: {
       /** @default false */
       layouts: boolean,

       /** @default true */
       pages: boolean,

       /** @default true */
       commons: boolean,
    },

    /** @default "auto" */
    corejs: string,

    babel: {
       /** @default false */
       configFile: boolean,

       /** @default false */
       babelrc: boolean,

       presets: any,

       /** @default true */
       cacheDirectory: boolean,
    },

    transpile: Array<any>,

    postcss: {
       preset: {
           /** @default 2 */
           stage: number,
       },
    },

    html: {
       minify: {
           /** @default true */
           collapseBooleanAttributes: boolean,

           /** @default true */
           decodeEntities: boolean,

           /** @default true */
           minifyCSS: boolean,

           /** @default true */
           minifyJS: boolean,

           /** @default true */
           processConditionalComments: boolean,

           /** @default true */
           removeEmptyAttributes: boolean,

           /** @default true */
           removeRedundantAttributes: boolean,

           /** @default true */
           trimCustomFragments: boolean,

           /** @default true */
           useShortDoctype: boolean,
       },
    },

    template: any,

    templates: Array<any>,

    watch: Array<any>,

    devMiddleware: {
       /** @default "none" */
       stats: string,
    },

    hotMiddleware: any,

    vendor: {
       "$meta": {
           /** @default "vendor has been deprecated since nuxt 2" */
           deprecated: string,
       },
    },

    stats: {
       /** @default [{},{},{}] */
       excludeAssets: Array<any>,
    },

    /** @default true */
    friendlyErrors: boolean,

    additionalExtensions: Array<any>,

    warningIgnoreFilters: Array<any>,

    /** @default false */
    followSymlinks: boolean,
  },

  messages: {
    /** @default "Loading..." */
    loading: string,

    /** @default "This page could not be found" */
    error_404: string,

    /** @default "Server error" */
    server_error: string,

    /** @default "Nuxt" */
    nuxtjs: string,

    /** @default "Back to the home page" */
    back_to_home: string,

    /** @default "An error occurred in the application and your page could not be served. If you are the application owner, check your logs for details." */
    server_error_details: string,

    /** @default "Error" */
    client_error: string,

    /** @default "An error occurred while rendering the page. Check developer tools console for details." */
    client_error_details: string,
  },

  render: {
    bundleRenderer: {
       shouldPrefetch: () => any,

       shouldPreload: () => any,

       /**
        * enabled by default for development
        * @default true
       */
       runInNewContext: boolean,
    },

    crossorigin: any,

    /** @default true */
    resourceHints: boolean,

    ssr: any,

    /** @default false */
    ssrLog: boolean,

    http2: {
       /** @default false */
       push: boolean,

       /** @default null */
       shouldPush: any,

       /** @default null */
       pushAssets: any,
    },

    static: {
       /** @default true */
       prefix: boolean,
    },

    compressor: {
       /** @default 0 */
       threshold: number,
    },

    etag: {
       /** @default false */
       hash: boolean,

       /** @default false */
       weak: boolean,
    },

    /** @default false */
    csp: boolean,

    dist: {
       /** @default false */
       index: boolean,

       /** @default "1y" */
       maxAge: string,
    },

    fallback: {
       dist: any,

       static: {
           /** @default true */
           skipUnknown: boolean,

           handlers: {
                /** @default false */
                ".htm": boolean,

                /** @default false */
                ".html": boolean,
           },
       },
    },
  },

  router: {
    /** @default "history" */
    mode: string,

    /** @default "/" */
    base: string,

    /** @default true */
    _routerBaseSpecified: boolean,

    routes: Array<any>,

    /** @default "-" */
    routeNameSplitter: string,

    middleware: Array<any>,

    /** @default "nuxt-link-active" */
    linkActiveClass: string,

    /** @default "nuxt-link-exact-active" */
    linkExactActiveClass: string,

    /** @default false */
    linkPrefetchedClass: boolean,

    /** @default null */
    extendRoutes: any,

    /**
     * 
     * @deprecated router.scrollBehavior` property is deprecated in favor of using `~/app/router.scrollBehavior.js` file, learn more: https://nuxtjs.org/api/configuration-router#scrollbehavior
    */
    scrollBehavior: any,

    /** @default false */
    parseQuery: boolean,

    /** @default false */
    stringifyQuery: boolean,

    /** @default false */
    fallback: boolean,

    /** @default true */
    prefetchLinks: boolean,

    /** @default true */
    prefetchPayloads: boolean,

    trailingSlash: any,
  },

  server: {
    /** @default false */
    https: boolean,

    /** @default 3000 */
    port: number,

    /** @default "localhost" */
    host: string,

    socket: any,

    timing: () => any,
  },

  cli: {
    badgeMessages: Array<any>,

    /** @default "green" */
    bannerColor: string,
  },

  generate: {
    /** @default "/home/pooya/tmp/framework/packages/kit/<rootDir>/dist" */
    dir: string,

    routes: Array<any>,

    exclude: Array<any>,

    /** @default 500 */
    concurrency: number,

    /** @default 0 */
    interval: number,

    /** @default true */
    subFolders: boolean,

    /** @default "200.html" */
    fallback: string,

    /** @default true */
    crawler: boolean,

    /** @default true */
    manifest: boolean,

    /** @default true */
    nojekyll: boolean,

    cache: {
       ignore: Array<any>,

       globbyOptions: {
           /** @default true */
           gitignore: boolean,
       },
    },

    staticAssets: {
       /** @default "static" */
       dir: string,

       /** @default "/home/pooya/tmp/framework/packages/kit/<rootDir>/dist" */
       base: string,

       /** @default "" */
       versionBase: string,

       /** @default "1617984167" */
       version: string,
    },
  },
}

declare type NuxtHook<Arg1 = any> = (arg1?: Arg1, ...args: any) => Promise<void> | void;
interface NuxtHooks {
    [key: string]: NuxtHook;
    'modules:before': NuxtHook<Nuxt>;
    'modules:done': NuxtHook<Nuxt>;
    'ready': NuxtHook<Nuxt>;
}
declare type NuxtHookName = keyof NuxtHooks;

interface Nuxt {
    options: NuxtOptions;
    hooks: {
        hook(hookName: NuxtHookName, callback: NuxtHook): any;
        callHook<T extends string>(hookbname: T, ...args: Parameters<NuxtHooks[T]>): any;
        addHooks(hooks: Partial<NuxtHooks>): any;
    };
    hook: Nuxt['hooks']['hook'];
    callHook: Nuxt['hooks']['callHook'];
    server?: any;
}

declare class ModuleContainer {
    nuxt: Nuxt;
    options: NuxtOptions;
    constructor(nuxt: Nuxt);
    private _call;
    ready(): Promise<void>;
    addVendor(): void;
    addTemplate(tmpl: TemplateOpts | string): {
        src: string;
        dst: string;
        options: Record<string, any>;
    };
    addPlugin(tmpl: PluginTemplateOpts): void;
    addLayout(tmpl: TemplateOpts, name: string): void;
    addErrorLayout(dst: string): void;
    addServerMiddleware(middleware: any): void;
    extendBuild(fn: any): void;
    extendRoutes(fn: any): void;
    requireModule(moduleOpts: any): Promise<void>;
    addModule(moduleOpts: any): Promise<void>;
}

interface ModuleMeta {
    name?: string;
    configKey?: string;
    [key: string]: any;
}
declare type ModuleOptions = Record<string, any>;
interface LegacyNuxtModule {
    (this: ModuleContainer, inlineOptions?: ModuleOptions): void | Promise<void>;
    meta?: ModuleMeta;
}
interface NuxtModule<T extends ModuleOptions = any> extends ModuleMeta {
    defaults?: T;
    setup?: (this: null, resolvedOptions: T, nuxt: Nuxt) => void | Promise<void>;
    hooks?: Partial<NuxtHooks>;
}
declare type ModuleSrc = string | NuxtModule | LegacyNuxtModule;
interface ModuleInstallOptionsObj {
    src: ModuleSrc;
    meta: ModuleMeta;
    options: ModuleOptions;
    handler: LegacyNuxtModule;
}
declare type ModuleInstallOptions = ModuleSrc | [
    ModuleSrc,
    ModuleOptions?
] | Partial<ModuleInstallOptionsObj>;
interface TemplateOpts {
    filename?: string;
    fileName?: string;
    options?: Record<string, any>;
    src: string;
}
interface PluginTemplateOpts extends TemplateOpts {
    /** @deprecated use mode */
    ssr?: boolean;
    mode?: 'all' | 'server' | 'client';
}

interface TypedConfigSchema extends ConfigSchema {
    hooks: NuxtHooks;
    modules: ModuleInstallOptions[];
    buildModules: ModuleInstallOptions[];
}
interface NuxtOptions extends TypedConfigSchema {
}
declare type DeepPartial<T> = {
    [P in keyof T]?: DeepPartial<T[P]> | T[P];
};
interface NuxtConfig extends DeepPartial<TypedConfigSchema> {
}

interface LoadNuxtConfigOptions {
    rootDir?: string;
    configFile?: string;
    config?: any;
}
declare function loadNuxtConfig(opts: LoadNuxtConfigOptions): NuxtOptions;

interface LoadDotEnvOptions {
    rootDir: string;
    dotenvFile: string;
    expand: boolean;
    env: typeof process.env;
}
declare function loadEnv(rootDir: any): Promise<void>;
declare function loalDotenv(opts: LoadDotEnvOptions): Promise<any>;

declare const nuxtCtx: unctx.UseContext<Nuxt>;
declare const useNuxt: () => Nuxt;
declare function defineNuxtConfig(config: NuxtConfig): NuxtConfig;

declare function defineNuxtModule<OptionsT extends ModuleOptions>(input: NuxtModule<OptionsT> | ((nuxt: Nuxt) => NuxtModule<OptionsT>)): LegacyNuxtModule;

declare function installModule(nuxt: Nuxt, installOpts: ModuleInstallOptions): Promise<void>;

declare function addTemplate(tmpl: TemplateOpts | string): {
    src: string;
    dst: string;
    options: Record<string, any>;
};
declare function addPlugin(tmpl: PluginTemplateOpts): void;
declare function addLayout(tmpl: TemplateOpts, name: string): void;
declare function addErrorLayout(dst: string): void;
declare function addServerMiddleware(middleware: any): void;
declare function extendBuild(fn: any): void;
declare function extendRoutes(fn: any): void;

interface ResolveModuleOptions {
    paths?: string[];
}
interface RequireModuleOptions extends ResolveModuleOptions {
    native?: boolean;
    clearCache?: boolean;
    interopDefault?: boolean;
}
declare function isNodeModules(id: string): boolean;
declare function clearRequireCache(id: string): void;
declare function scanRequireTree(id: string, files?: Set<string>): Set<string>;
declare function getRequireCacheItem(id: string): NodeModule;
declare function requireModulePkg(id: string, opts?: RequireModuleOptions): any;
declare function resolveModule(id: string, opts?: ResolveModuleOptions): string;
declare function tryResolveModule(path: string, opts?: ResolveModuleOptions): string;
declare function requireModule(id: string, opts?: RequireModuleOptions): any;
declare function tryRequireModule(id: string, opts?: RequireModuleOptions): any;

interface ResolveOptions {
    base?: string;
    alias?: Record<string, string>;
    extensions?: string[];
}
declare function resolveAlias(path: string, alias: ResolveOptions['alias']): string;
declare function tryResolvePath(path: string, opts?: ResolveOptions): string;

export { LegacyNuxtModule, LoadDotEnvOptions, LoadNuxtConfigOptions, ModuleContainer, ModuleInstallOptions, ModuleInstallOptionsObj, ModuleMeta, ModuleOptions, ModuleSrc, Nuxt, NuxtConfig, NuxtHook, NuxtHookName, NuxtHooks, NuxtModule, NuxtOptions, PluginTemplateOpts, RequireModuleOptions, ResolveModuleOptions, ResolveOptions, TemplateOpts, TypedConfigSchema, addErrorLayout, addLayout, addPlugin, addServerMiddleware, addTemplate, clearRequireCache, defineNuxtConfig, defineNuxtModule, extendBuild, extendRoutes, getRequireCacheItem, installModule, isNodeModules, loadEnv, loadNuxtConfig, loalDotenv, nuxtCtx, requireModule, requireModulePkg, resolveAlias, resolveModule, scanRequireTree, tryRequireModule, tryResolveModule, tryResolvePath, useNuxt };
